#!/usr/bin/env python
"""Handle Anisotropy of Magnetic Susceptibility data (KLY-2)

Hext - statistics for Anisotropy
S_Matrix - the diagonalized s matrix

Right now this just wraps k15_hext, k15_s, s_eigs, etc

# $Id: ams.py,v 1.17 2005/11/21 16:44:41 schwehr Exp $

######################################################################
#     Copyright (C) 2005  Kurt Schwehr
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
######################################################################

"""
__version__ = '$Revision: 1.17 $'.split()[1] # See man ident
__date__ = '$Date: 2005/11/21 16:44:41 $' # FIX: pull out just the date
__author__ = 'Kurt Schwehr'

import popen2,string,math

#EXIT_SUCCESS=0
#EXIT_FAILURE=1


def getShapeName(F,F12,F23,confidence=0.95):
    """
    Use the F-Test and a confidence level to give a fabric shape name
    """
    if F<getF(confidence): return "isotropic"
    if F12>getF12(confidence) and F23>getF23(confidence): return "triaxial"
    if F12>getF12(confidence): return "prolate"
    if F23>getF23(confidence): return "oblate"

    # This case does happen sometimes
    print "WARNING: You're screwed... I think you picked up some antimatter"
    return "unknown"

def getF(confidence=.95):
    """
    Return the confidence value for the F-test for a 15 position
    measurement of AMS 

    Posted about bug on Nov 3 2003 on the news group comp.lang.python
    bug is gone!  11/2005

    > getF(.95)
    3.4817

    FIX: Need to VERIFY the F statistics usage!!!!!
    
    FIX: it is not 2,7... what is it?  is it 9,5?
    FIX: go back to my 11/2003 notes to figure out what it is

    Is it this???
    In [2]: distributions.f.ppf(0.95,5,9) 
    Out[2]: 3.4816586539
    """
    threshold=None
    if confidence==.95:
        threshold=3.4817
    else:
        print "WARNING:  need to verify getF is using the proper degrees of freedom!"
        threshold = distributions.f.ppf(confidence,5,9) 
    return threshold

def getF12(confidence=.95):
    """
    If F12 value is greated than this number, then the max and intermediate
    eigenvectors are distinguishable.
    
    > get_F12(.95)
    4.25649472909
    # Which equals the 4.2565 that is published

    from rpy import *
    > print r.qf(0.95,2,9)
    4.256495
    > print r.pf(4.256495,2,9)
    0.95 

    import scipy.stats
    > scipy.stats.distributions.f.ppf(0.95,2,9)
    4.25649472909

    FIX: Need to VERIFY the F statistics usage!!!!!

    FIX: why is scipy broke?
    """
    if confidence==.95:
        return 4.25649472909

    assert(False)
    from scipy.stats import distributions
    threshold = distributions.f.ppf(0.95,2,9)
    return threshold

def getF23(confidence=.95):
    """SEE ALSO: getF12"""
    return getF12(confidence)


class S_Matrix:
    """ k15_s wrapper """
    def __init__(self,k15_lines):
        """ Take 4 lines of k15 data and fill in the object"""
        assert 4 == len(k15_lines)

        # Use pmag >= 1.9  k15_hext for now.  FIX to use all python code
        # FIX: make sure the lines all have \n at the end
        fin,fout=popen2.popen2("k15_s")
        for l in range(4):
            fout.write(k15_lines[l])
        fout.close()
        line = fin.readline()
        self.values = line.split()
        for i in range(7):
            self.values[i] = float(self.values[i])
        self.sigma = self.values[6]


class Hext:
    """ Handle k15_hext output """
    maxpos = 0
    intpos = 1
    minpos = 2
    def __init__(self,k15_lines,volumeCC=10,dataInMicroSI=True):
        """Take 4 lines of k15 Kappabridge data and create a Hext stats object

        volumeCC - Volume of the sample.  The KLY-2 assume a volume of 10 cm^3

        dataInMicroSI - The KLY-2 kappa bridge returns values in microSI (1e-6).  If data
                        is not in micro SI, set this to false.  Only affects the bulk susc.

        WARNING: I think the values set in this class are probably a
        mixture of strings and numbers.  Needs to be cleanedup so all
        strings that really are numbers get converted to numbers.

        FIX: make this pure python
        """
        assert 4 == len(k15_lines)

        # Use k15_hext for now.  FIX to use all python code
        # FIX: make sure the lines all have \n at the end
        fin,fout=popen2.popen2("k15_hext")
        for l in range(4):
            fout.write(k15_lines[l])
        fout.close()
        lines=[]
        for l in range(5):
            lines.append(fin.readline())
        #print lines
        # d is for dummy positions of things we want to just drop
        self.sampleName,d,d,d,self.bulk_susc = lines[0].split()
        #print self.sampleName, self.bulk_susc

        if dataInMicroSI:
            # Convert from microSI (kappa bridge raw values) to SI
            self.bulk_susc = float(self.bulk_susc) * 1e-6

        if volumeCC != 10:
            # Normalize volume to actual amount in a cube or cylinder
            self.bulk_susc = self.bulk_susc * (10. / volumeCC)

        d,d, self.F, d,d, self.F12,d,d,self.F23 = lines[1].split()
        self.F = float(self.F)
        self.F12 = float(self.F12)
        self.F23 = float(self.F23)
        #print self.F,self.F12,self.F23
        self.tau=[]; self.dec=[]; self.inc=[];
        self.Eij=[]; self.Eij_dec=[]; self.Eij_inc=[];
        self.Eik=[]; self.Eik_dec=[]; self.Eik_inc=[];

        for i in range(3):
            tau, dec, inc, Eij, Eij_dec, Eij_inc, Eik, Eik_dec, Eik_inc = lines[i+2].split()
            self.tau.append(float(tau))
            self.dec.append(float(dec))
            self.inc.append(float(inc))

            self.Eij.append(float(Eij))
            self.Eij_dec.append(float(Eij_dec))
            self.Eij_inc.append(float(Eij_inc))

            self.Eik.append(float(Eik))
            self.Eik_dec.append(float(Eik_dec))
            self.Eik_inc.append(float(Eik_inc))


    def getLparam(self):
        """L shape parameter is Tau1/Tau2  (max/int eigenvalues) - aka Lineation

        Balsley and Buddington [1960]
        """
        L = self.tau[0]/self.tau[1]
        assert(L>=1.0)
        return L
    
    def getFparam(self):
        """F shape parameter is Tau2/Tau3  (int/min eigenvalues) - aka Foliation

        Stacey et al. [1960]
        """
        F = self.tau[1]/self.tau[2]
        assert(F>=1.0)
        return self.tau[1]/self.tau[2]

    def getEta(self,index):
        """Eta is the ln(tau)   eta1 = ln(tau1)"""
        assert(1<=index)
        assert(3>=index)
        eta = math.log(self.tau[index-1])
        return eta
    
    def getPprime(self):
        """P' shape paramter is hard to write in ascii text

        'Corrected' Anisotropy (Jelinek [1981])
        """
        from math import exp,sqrt
        eta1 = self.getEta(1); eta2 = self.getEta(2); eta3 = self.getEta(2);
        etaAve = (eta1+eta2+eta3)/3.
        a = (eta1-etaAve)*(eta1-etaAve)
        b = (eta2-etaAve)*(eta2-etaAve)
        c = (eta3-etaAve)*(eta3-etaAve)
        Pprime = exp(sqrt(2*(a+b+c)))

        return Pprime
        

# Test code
if __name__ == '__main__':
    print "Testing...  FIX: would be good to use unittest!"
    
    k15_lines=[]
    k15_lines.append( "bp04-6gw-s1-141.2	schwehr	1/3/05	4:33 PM	 1716	 -4.75\n")
    k15_lines.append( "9.3800E+1	9.2950E+1	9.3550E+1	9.3550E+1	9.2950E+1\n" )
    k15_lines.append( "8.9150E+1	8.9100E+1	9.2900E+1	8.9150E+1	8.8900E+1\n" )
    k15_lines.append( "8.9950E+1	8.8750E+1	8.5100E+1	8.9950E+1	8.8800E+1\n" )
    
    hext = Hext(k15_lines)
    print "Tau 1 (max) = ", hext.tau[Hext.maxpos]
    print "Tau 2 (int) = ", hext.tau[Hext.intpos]
    print "Tau 3 (min) = ", hext.tau[Hext.minpos]
