#!/usr/bin/env python

"""
Stuff for working with the specific setup for the BPSIO Aug 2004
Cruise to the Santa Barbara Basin: Schwehr, Babcock, Driscoll, Kent,
and Jim from BP.

     Copyright (C) 2005  Kurt Schwehr

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

$Id: bpsio.py,v 1.13 2005/10/29 15:31:06 schwehr Exp $
"""

import unittest
import sqlite # for DepthLookup

dbFileName = "/Users/schwehr/projects/xcore/data/BPSIO-04/bpsio04.db"

class SampleName:
    """ Parse a Kurt style pmag sample cube name """
    def __init__(self,nameString=""):
        if 0==len(nameString):
            # Make a blank object
            self.cruise="None"
            self.coreNum=-1
        else:
            #self.cruise,core,section,self.depthOffset=nameString.split("-")
            lst = nameString.split("-")

            self.cruise=lst[0]
            # FIX: handle core numbers greater than 9
            self.coreNum=int(lst[1][0])
            self.coreType=lst[1][1]
            self.coreHalf=lst[1][2]

            self.section=int(lst[2][1])
            self.depthOffset=float(lst[3])
            # FIX: check for optional part
            if 4==len(lst):
                # There was no L,C,R tag for left, center, right
                self.horizPart=None
            else:
                self.horizPart=lst[4]

    def __str__(self):
        """ Reconstruct the original sample name string """
        tmpStr=self.cruise+"-"+("%d"%self.coreNum)+self.coreType+self.coreHalf
        tmpStr+="-s" + ("%d" % self.section) + "-"
        o = self.depthOffset
        if ( (round(o)-0.01<o) & (round(o)+0.01>o) ):
            tmpStr+="%03d" % int(o)
        else:
            tmpStr+="%03.1f" % o
        if (self.horizPart):
            tmpStr+="-"+self.horizPart
        else:
            pass # Then just drop it
        return tmpStr


class DepthLookup:
    def __init__(self, db_cx):
        """ Lookup the section table and make a dictionary.  Loads from the database section table """
        cu = db_cx.cursor()
        cu.execute ("SELECT cruise,corenum,section,sectopdepth FROM sections")
        self.table = {}
        for row in cu.fetchall():
            key = row.cruise + "-" + str(row.corenum) + "-" + str(row.section)
            #print "key=",key
            self.table[key] = row.sectopdepth

    def lookup(self, sampleNameStr):
        """ return the depth in cm """
        sn = SampleName(sampleNameStr)
        
        key = sn.cruise + "-" + str(sn.coreNum) + '-' + str(sn.section)
        depth = self.table[key] + sn.depthOffset
        #print "Found: ",key, depth, "(",self.table[key],"+",sn.depthOffset,")"
        return depth


def getCoreLength(db_cx, corenum, coretype='g'):
    """
    FIX: make coretype actually be used
    """
    cu = db_cx.cursor()
    #cu.execute("SELECT sectopdepth,sectionlength FROM sections WHERE corenum='"+str(corenum)+"' AND coretype='"+coretype+"';")
    cu.execute("SELECT sectopdepth,sectionlength FROM sections WHERE corenum='"+str(corenum)+"' ORDER BY section DESC;")
    # FIX: only fetch the first entry
    row = cu.fetchone()
    length = float(row.sectopdepth)+float(row.sectionlength)
    #for row in cu.fetchall():
    #    print row
    return length


def getDepth(db_cx, samplename, depthTable='mag'):
    """
    Lookup the depth of a sample from a table.

    PARAMETERS:
    db_cx      - sqlite database connection
    samplename - find the depth for this sample
    depthTable - the table that has both samplename and depth fields

    RETURNS:
    depth in cm
    """
    cu = db_cx.cursor()
    cu.execute("SELECT depth FROM "+depthTable+" WHERE samplename='"+samplename+"';")
    row = cu.fetchone()
    depth = row['depth']
    return depth


def getRot(db_cx,core,section):

    """ return combined halfrot and decrot - this is the total
    rotation that should be applied to a core.

    db_cx   -  sqlite database handle
    core    -  corenumber (integer)
    section -  which section of the core.  1 is the top
    """
    
    cu = db_cx.cursor()
    searchStr = "SELECT halfrot,decrot FROM sections "
    searchStr += "WHERE corenum="+str(core)+" and section="+str(section)+";"
    cu.execute(searchStr)
    row = cu.fetchone()
    #print "rot row", core, section, row
    return (row['halfrot'] + row['decrot'])


def getCoreDescription(db_cx,corenum):
    """ return the string for the one line core location / context description

    db_cx    --- the sqlite database handle
    corenum  --- numeric id for the core (1..6)
    """
    cu = db_cx.cursor()
    cu.execute("SELECT descr FROM CoreLoc where corenum="+str(corenum)+";")
    row = cu.fetchone()
    return (row['descr'])
    

def fixDecRange(dec):
    """Make the Dec be between -180 and 180 for better plotting"""
    dec = dec % 360
    if (-180>dec): dec += 360
    if (180<dec): dec -=360
    assert -180<=dec
    assert dec<=180
    return dec

########################################
def getCoreLength(db_cx, corenum):
    """

    Compute the length of a core from the last section top depth and
    the length of the section.

    FIX: do a better SQL query that only gets the 1st entry

    This is not quite it...
    select max(section),sectopdepth,sectionlength FROM sections WHERE corenum=1 group by sectopdepth,sectionlength;


    """
    cu = db_cx.cursor()
    searchStr  = "SELECT section,sectopdepth,sectionlength FROM sections"
    searchStr += " WHERE corenum="+str(corenum)
    searchStr += " ORDER BY section DESC"
    searchStr += ";"
    cu.execute(searchStr)
    row = cu.fetchone()
    return (float(row['sectopdepth'])+float(row['sectionlength']))

######################################################################
def getWaterWeightPercent(db_cx, corenum, type=None, half=None):
    """
    Get the weight percent of water for a core

    Arguments

    db_cx - database connection, sqlite
    corenum - integer core number
    type - type of core (e.g. 'g' for gravity)
    half - which half of the core (e.g. 'w' for working, 'a' for archive)

    Returns:

    A list of (depth,waterPercent) pairs.  Depth in cm
    
    """
    cu = db_cx.cursor()

    names=[]
    dry=[]
    wet=[]
    water=[]
    weightPercent=[]
    cu.execute("SELECT weights.samplename,weight FROM weights,ams WHERE weights.samplename=ams.samplename AND status='dry' ORDER BY depth;")
    # FIX: use something like this instead?
    #     AND samplename=bp04-"+str(corenum)+"%
    matchStr = "bp04-"+str(corenum)
    if type: matchStr += type
    if half: matchStr += half
    for row in cu.fetchall():
        nm = row['weights.samplename']
        if -1 == nm.find(matchStr): continue  # is not a part of what we want
        #names.append(row['samplename'])
        names.append(nm)
        dry.append(row['weight'])
    for name in names:
        searchStr = "SELECT weights.samplename,weight FROM weights,ams WHERE weights.samplename=ams.samplename AND status='wet' AND weights.samplename='"+name+"' ORDER BY depth;"
        cu.execute(searchStr)
        row = cu.fetchone()
        wet.append(row['weight'])
    # handle case where there are no samples dry

    for i in range(len(names)):
        water.append(wet[i]-dry[i])
        weightPercent.append( (wet[i]-dry[i])/wet[i] )

    depthTable = DepthLookup(db_cx)

    depths=[]
    for i in range(len(names)):
        depths.append(depthTable.lookup(names[i]))

    returnList = []
    for i in range(len(weightPercent)):
        returnList.append((depths[i],weightPercent[i]))

    return returnList


######################################################################
# TESTING
######################################################################
def setDecRange(dec,center):
    """ Convert a degree direction to a specified center range +/- 180 degrees

    dec    --- direction to be modified in degrees
    center --- center of the 360 degree range

    e.g. center=180 will give 0..360, while center=0 will give +/- 180 degrees
    """
    min = center - 180
    max = center + 180
    dec = dec % 360
    while dec > max:  dec -=360
    while dec < min:  dec +=360
    return dec


class BpsioTests(unittest.TestCase):
    """
    Some of the test cases here may seem trivial, but it is good to
    cover all the bases!

    Run the test cases by doing:

       ./bpsio.py
    """
    def test_setDecRange_0_360(self):
        """ Test the simplest case for 0...360 deg"""
        self.failUnless(0==setDecRange(0,180))
        self.failUnless(90==setDecRange(90,180))
        self.failUnless(180==setDecRange(180,180))
        self.failUnless(270==setDecRange(270,180))
        self.failUnless(359==setDecRange(359,180))
        self.failUnless(0==setDecRange(360,180))

    def test_setDecRange_0_360_large(self):
        """Make sure that wrapping workings in the 0..360 case"""
        self.failUnless(359==setDecRange(360+359,180))
        self.failUnless(359==setDecRange((2*360)+359,180))

    def test_setDecRange_0_360_neg(self):
        """Make sure that negatives popup right"""
        self.failUnless(359==setDecRange(-1,180))
        self.failUnless(180==setDecRange(-180,180))

    def test_setDecRange_0_360_neg_large(self):
        """Check wrapping in the negative direction"""
        self.failUnless(359==setDecRange(-1-360,180))
        self.failUnless(359==setDecRange(-1-(2*360),180))

    def test_setDecRange_neg180_180(self):
        """Basic check for -180...180, 0 centered range"""
        self.failUnless(0==setDecRange(0,0))
        self.failUnless(-1==setDecRange(-1,0))
        self.failUnless(1==setDecRange(1,0))


# Test code
if __name__ == '__main__':
    print "Testing..."
    testname1="bp04-6gw-s1-064.5-L"
    testname2="bp04-6gw-s1-064.5"
    testname3="bp04-6gw-s1-064"
    name0=SampleName()
    name=SampleName(testname1); print name
    name=SampleName(testname2); print name
    name=SampleName(testname3); print name

    unittest.main()

    

