"""
Command utility to examine a SEGY file

     Copyright (C) 2005  Kurt Schwehr

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

Originally in segy-py version 0.30

How do I assemble queries like this?

    SELECT COUNT(samplename) AS count FROM \\
        (SELECT DISTINCT(samplename) AS samplename FROM \\
         ams WHERE corenum=1 AND coretype='p');

TODO:
  * subqueries
  * make a super class so that inserts and selects can verify based on the create str
  * take the super class info from the database?

"""
__version__ = '$Revision: 1.5 $'.split()[1] # See man ident
__date__ = '$Date: 2005/11/21 16:45:37 $'.split()[1] # FIX: pull out just the date
__author__ = 'Kurt Schwehr'

import sys

class sqlselect:
    """
    Construct an sql select query

    Sometimes it just gets ugly having all that comma and WHERE AND
    logic in there.  This code takes care of that
    """
    def __init__(self):
        self.fields = []
        self.where = []
        self.limit = None
        self.from_tables = []
        self.orderby = None
        self.desc = False # descending sort if true
        return

    def setorderby(self,field,desc=False):
        "Make the returned rows come in some order"
        if str != type(field): print "ERROR: fix throw type exception"
        self.orderby = field
        self.desc = desc
        return

    def addfield(self,fieldname):
        "Add a field name to return"
        if str != type(fieldname): print "ERROR: fix throw type exception"
        self.fields.append(fieldname)
        return

    def addwhere(self,boolTest):
        " Add expressions to chain together with ANDs"
        if str != type(boolTest):
            print "ERROR: fix throw type exception"
        self.where.append(boolTest)
        return

    def addfrom(self,tableName):
        "Which tables the query will pull from"
        if str != type(tableName):
            print "ERROR: fix throw type exception"
        self.from_tables.append(tableName)
        return

    def setlimit(self,numOfItems):
        "Set the maximum number of items to return"
        if int != type(numOfItems):
            print "ERROR: fix throw type exception"
        self.limit = numOfItems
        return

    def __str__(self):
        "Return the query as a string"
        if len(self.fields) < 1: print "ERROR: Must specify at least one from!\n  FIX: throw some exception?"
        s = 'SELECT '
        for i in range (len(self.fields)-1):
            s += self.fields[i]+','
        s += self.fields[-1] + ' '

        if len(self.from_tables)<1: print "ERROR: fix throw some exception"
        s += 'FROM '
        for i in range (len(self.from_tables)-1):
            s += self.from_tables[i]+','
        s += self.from_tables[-1]

        if (len(self.where)>0): s += ' WHERE '
        for i in range (len(self.where)-1):
            s += self.where[i]+' AND '
        if (len(self.where)>0): s += self.where[-1]

        if (None != self.orderby):
            s += ' ORDER BY ' + self.orderby
            if self.desc: s += ' DESC'

        if (None != self.limit):
            s += ' LIMIT ' + str(self.limit)

        s += ';'
        return s

class sqlcreate:
    """
    Helper for building create SQL commands.

    FIX: add type checking
    """

    def __init__(self,table):
        """Kick it off with no fields

        table - which table are we going to insert into"""
        self.table = table
        self.fields = []
        self.types = []
        return

    def add(self,field,typeStr):
        """
        Unchecked field.  Provide the field and type all in one.  Use
        this if nothing matches what you need.

        e.g.:
        create.add('corenumber','INTEGER')
        create.add('username','VARCHAR(40)')
        create.add('id','INTEGER PRIMARY KEY')
        """
        self.fields.append(field)
        self.types.append(typeStr)
        return

    def addInt(self,field):
        self.fields.append(field)
        self.types.append("INTEGER")

    def addReal(self,field):
        self.fields.append(field)
        self.types.append("REAL")

    def addVarChar(self,field,length):
        self.fields.append(field)
        self.types.append("VARCHAR("+str(length)+")")
        

    def __str__(self):
        "Return the SQL string for the table creation"
        cstr = 'CREATE TABLE '+self.table+' ( '
        assert (len(self.fields)==len(self.types))
        for i in range(len(self.fields)-1):
            cstr += str(self.fields[i])+' '+str(self.types[i])+', '
        cstr += str(self.fields[-1])+' '+str(self.types[-1])
        cstr += ' ); '
        return cstr

class sqlinsert:
    """
    Help create an SQL insert statement for injecting data into a database.  Wee!

    FIX: provide some sort of validation, maybe with the CREATE string or class?
    """
    def __init__(self,table):
        """Kick it off with no fields

        table - which table are we going to insert into"""
        self.table = table
        self.fields = []
        self.values = []
        return
    
    def __str__(self):
        "Return the SQL string for the insert"
        if 0==len(self.fields):
            print "WARNING: empty insert.  returning empty string"
            return ""  # FIX: throw exception and a hissy fit
        
        s = 'INSERT INTO ' + self.table + ' '

        assert(len(self.fields)==len(self.values))
        s1 = ''
        s2 = ''

        # FIX: insert the 1st without a leading ','
        s1 += str(self.fields[0])
        if str == type(self.values[0]): s2 += '"'+str(self.values[0])+'"'
        else: s2 += str(self.values[0])

        for i in range(1,len(self.fields)):
            s1 += ','+str(self.fields[i])
            if str == type(self.values[i]):
                s2 += ',"'+str(self.values[i])+'"'
            else:
                s2 += ','+str(self.values[i])
        s += '(' + s1 + ') VALUES (' + s2 + ');'
        return s

    def add(self,field,value):
        """Add a field value pair to the insert

        Integers and floats should NOT be converted to strings.
        """

        self.fields.append(field)
        self.values.append(value)
        return

######################################################################
#import unittest
import datetime

def sqlInsertStrFromList (table,aList):
    """ Take a list and make an insert string.  This works with
    dictionaries too.  Here is a quick example:

    aList = [('one',1),('2','two'),('threepoint',3.)]
    insertStr = sql_util.sqlInsertStrFromList('myTable',aList)
    """
    ins = "insert into " + table + " ("
    first=True
    for pair in aList:
        key = pair[0]
        if first: first=False
        else:     ins+=","
        ins+=str(key)
    ins += ") values ("
    first=True
    for pair in aList:
        value = pair[1]
        if first: first=False
        else:     ins+=","
        # Make sure to quote all strings and timestamps
        # print type(value)
        # <type 'DateTime'>
        # What way is better to handle this?
        #if type(value) == str or type(value) == type(datetime()): ins+="'"
        if type(value)!=int and type(value)!=float: ins+="'"
        ins+=str(value)
        if type(value)!=int and type(value)!=float: ins+="'"
        #if type(value) == str or type(value) == type(datetime()): ins+="'"
    ins += ");"
    return ins

# FIX: bring in unittest from sql_util.py in BPSIO-04
