#!/usr/bin/env python

"""
Hangle X-Ray Diffraction data

     Copyright (C) 2005  Kurt Schwehr

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

Tested only with the SIO AF Scintag XDS 2000 running the DMSNT
software (version unknown).  This software was developed with only 2
runs of the XRD, so I imaging it will break with other files.

SEE ALSO:

  http://www.matter.org.uk/diffraction/

  http://epswww.unm.edu/xrd/resources.htm

  Sample cards: http://www.icdd.com/resources/webpdf/webpdf.htm

DATABASE:

  What is the best way to build an open database of XRD specta?  It
  seems like a full database system is key.  Writing spectra in python
  dictories is clunky and will be slow when the database reaches any
  sort of reasonable size.

  It would be nice to also have some catagories that would help limit
  the number of materials to search... e.g. marine sediments

  If you want to contribute data, please send me a patch or you are
  welcome to copy this file and start up your own.

  Still need to have options to show the expected peaks for minerals.
"""
__version__ = '$Revision: 1.6 $'.split()[1] # See man ident
__date__ = '$Date: 2005/11/05 01:59:22 $'.split()[1]
__author__ = 'Kurt Schwehr'

import sys
from pmag_verbosity import BOMBASTIC,VERBOSE,TRACE,TERSE,ALWAYS
from pmag_kds_version import VERSION

database = {}

# http://www.icdd.com/resources/webpdf/corundumc.pdf
# Spectrum entries are d(A) Int h k l
# Is d(A) is the angle in radians???
database['corondum'] = {
    'wavelength':1.540562,
    'alt name':'Aluminum Oxide',
    'spectrum':[(3.4797,45,0,1,2),
              (2.5508,100,1,0,4),
              (2.3794,21,1,1,0),
              (2.1654,2,0,0,6),
              (2.0853,66,1,1,3),
              (1.9643,1,2,0,2),
              (1.7400,34,0,2,4),
              (1.6015,89,1,1,6),
              (1.5466,1,2,1,1),
              (1.5150,2,1,2,2),
              (1.5110,14,0,1,8),
              (1.4045,23,2,1,4),
              (1.3737,27,3,0,0),
              (1.3359,1,1,2,5),
              (1.2755,2,2,0,8),
              (1.2391,29,1,0,10),
              (1.2343,12,1,1,9),
              (1.1931,1,2,1,7),
              (1.1897,2,2,2,0),
              (1.1600,1,3,0,6),
              (1.1472,3,2,2,3),
              (1.1386,1,1,3,1), # How do I encode <1 for the Int column???
              (1.1256,2,3,1,2),
              (1.1241,3,1,2,8),
              (1.0990,9,0,2,10)
              ]
    }


def rows2cols(list):
    """
    convert a big list of rows to list of columns.

    [ [1,2], [3,4], [5,6] ] to  [ [1,3,5], [2,4,6] ]
    """
    newlist = []
    for i in range(len(list[0])): newlist.append([])
    for row in list:
        for i in range(len(list[0])):
            newlist[i].append(row[i])
    return newlist


class XRDcps:
    def __init__(self,filename,degreeOffset=0.0,verbosity=0):
        """
        load an XRD counts per second file from disk
        
        degreeOffset - apply this shift to the data to correct for gear issues
           SIO AF XRD needs -0.2 during Oct 2005
           Correction is applied in read_range
        
        FIX: make reading from a file optional
        FIX: make a validatation from a known file
        FIX: make reading fields be methods... read_string_field, read_deg_field, read_float_field
        FIX:   make reading fields verify the field name
        """
        self.degree_offset = degreeOffset
        self.file = open(filename,'r')
        f = self.file
        self.ID = self.read_field_string('ID')
        f.readline() # blank
        self.comment = f.readline()
        f.readline() # blank
        self.scan_type   = self.read_field_string('Scan Type')
        self.start_angle = self.read_field_degrees('Start Angle')
        self.stop_angle  = self.read_field_degrees('Stop Angle')
        #self.num_points  = int(f.readline().split(':')[1].split()[0])
        self.num_points  = self.read_field_int('Num Points')
        self.step_size   = self.read_field_float('Step Size')
        self.datafile_res= self.read_field_int('Datafile Res')
        self.scan_rate   = self.read_field_float('Scan Rate')
        self.scan_mode   = self.read_field_string('Scan Mode')
        self.wavelength  = self.read_field_float('Wavelength')

        f.readline() # blank
        self.read_field_blank('Diffractometer Optics')

        self.read_field_blank('Detector')
        self.detector_type = self.read_field_string('Type')
        self.detector_x2 = self.read_field_yesno('X2 Configuration')

        self.read_field_blank('Tube')
        self.tube_type = self.read_field_string('Type')
        self.tube_x2 = self.read_field_yesno('X2 Configuration')

        f.readline() # blank
        f.readline() # blank
        self.axes = []
        for i in range(10):
            self.axes.append(self.read_axis())

        self.read_until('Deg')
        f.readline()
        # FIX: handle multiple ranges
        self.rangeNum,self.rangeData = self.read_range()

        return



    def read_range(self):
        """
        CPS - counts per second
        ESD - CPS minus some sort of baseline

        uses self.degree_offset
        """
        line = self.file.readline()
        rangeStr,range = line.split()
        assert('Range'==rangeStr.strip())
        range = int(range)
        # FIX: handle multiple ranges
        line = self.file.readline()
        data = []
        while (3 == len(line.split())):
            deg,cps,esd = line.split()
            deg = float(deg) + self.degree_offset
            cps = float(cps)
            esd = float(esd)
            row = (deg,cps,esd)
            data.append(row)
            line = self.file.readline()

        return range,data
    
    def read_until(self,start_str):
        """FIX: cleanup"""
        line = self.file.readline()
        for i in range(100):
            #print i
            #print len(line),len(start_str)
            #print line.strip()
            #print line.strip()[:len(start_str)]
            if (len(line)<len(start_str) or line.strip()[:len(start_str)] != start_str): 
                line = self.file.readline()
            else: break
        

    def read_axis(self):
        line = 'None'
        axis = {}
        while line[0:4] != 'Axis': line = self.file.readline()
        axis['num'] = line.split('[')[1].split(']')[0]
        axis['selected'] = self.read_field_yesno('Selected')
        axis['iterative'] = self.read_field_yesno('Iterative')
        axis['start_angle'] = self.read_field_degrees('Start Angle',False)
        axis['stop_angle'] = self.read_field_degrees('Stop Angle',False)
        axis['step_size'] = self.read_field_degrees('Step Size',False)
        axis['oscillating'] = self.read_field_yesno('Oscillating')
        #print axis
        return(axis)
        
    def read_field_blank(self,field_name):
        line = self.file.readline()
        field_name_in,value = line.split(':')
        assert(field_name.strip()==field_name_in.strip())
        return
    
    def read_field_yesno(self,field_name):
        line = self.file.readline()
        field_name_in,value = line.split(':')
        assert(field_name.strip()==field_name_in.strip())
        value = value.strip().lower()
        if 'no'==value: return False
        if 'yes'==value: return False
        assert(False)

    def read_field_int(self,field_name):
        line = self.file.readline()
        field_name_in,value = line.split(':')
        assert(field_name.strip()==field_name_in.strip())
        value = int(value.split()[0])
        return value

    def read_field_float(self,field_name):
        line = self.file.readline()
        field_name_in,value = line.split(':')
        assert(field_name.strip()==field_name_in.strip())
        value = float(value.split()[0])
        return value

    def read_field_degrees(self,field_name,hasDeg=True):
        """ read a degrees field """
        line = self.file.readline()
        field_name_in,value = line.split(':')
        assert(field_name.strip()==field_name_in.strip())
        if hasDeg:
            assert ('deg.'==value.split()[1])
        value = float(value.split()[0])
        return value

    def read_field_string(self,field_name):
        """ Read a string field and verify the field name """
        line = self.file.readline()
        field_name_in,value = line.split(':')
        assert(field_name.strip()==field_name_in.strip())
        return value.strip()

    def plot(self,options, cps=True,esd=False, filename=None):
        """ Use matplot to display a graph

        options is where to put xmin,xmax,ymin,ymax
        """
        import pylab
        verbosity = options.verbosity
        if verbosity>TRACE:
            print options.xmin,options.xmax,options.ymin,options.ymax

        cols = rows2cols(self.rangeData)
        
        if cps: pylab.plot(cols[0],cols[1])
        hold = False
        if cps: hold=True
        if esd: pylab.plot(cols[0],cols[2],hold=hold)
        pylab.xlabel('Degrees')
        if cps and esd: pylab.ylabel('CPS and ESD')
        elif esd: pylab.ylabel('ESD')
        else: pylab.ylabel('CPS')
        pylab.title(str(self.ID))

        axes = None
        if options.xmin or options.ymin: axis = pylab.axes()
        if options.xmin:
            print 'setting x axis to ', (options.xmin,options.xmax)
            axis.set_xlim((options.xmin,options.xmax))
        if options.ymin: axis.set_ylim((options.ymin,options.ymax))

        if filename==None: pylab.show()
        else: pylab.savefig(filename)

    def printStrs(self):
        print self.ID
        #print self.comment
        print self.scan_type
        print self.start_angle
        print self.stop_angle
        print self.num_points
        print self.step_size
        print self.scan_rate
        print self.scan_mode
        print self.wavelength
        print 'Diffract optics detector:',self.detector_type, '  x2:',self.detector_x2
        print 'Diffract optics tube:    ',self.detector_type, '  x2:',self.detector_x2
        #print self.axes
        print self.rangeNum, len(self.rangeData)

def printTodo():
    """
    Things that need to be or could be done to this code:

    FIX: Better labeling of graphs
    FIX: Start building a database of minerals
    FIX: Allow inputing of sample preparation
    FIX: Parse the other file formats... counts, summary, raw, emf... and what is an emf
    FIX: make verbosity to something...
    FIX: and???
    """
    print printTodo.__doc__


if __name__ == "__main__":
    from optparse import OptionParser

    myparser = OptionParser(usage="%prog [options]",
                            version="%prog "+VERSION+' - '+__version__)

    myparser.add_option('--deg-offset',dest='offset',default=0.0,type='float',
                        help='Specify a machine offset if required.  SIO ESD is off by -0.2 deg'
                        +' in Oct 2005 [default: %default]')
    myparser.add_option('--cps-file',dest='cpsFile',default=None,type='string',
                        help='Read a counts per second formated file')

    myparser.add_option('--no-cps',dest='cps',default=True,action='store_false',
                        help='Do not graph counts per second')
    myparser.add_option('--esd',dest='esd',default=False,action='store_true',
                        help='Graph the esd values from a cps file')

    myparser.add_option('--show',dest='show',default=False,action='store_true',
                        help='Show a matplotlib graph of the data')

    myparser.add_option('--plot-file',dest='plotFile',default=None,type='string',
                        help='Save plot to a file.  Filename extension determines the filetype.  '+
                        'e.g. .svg, .png, .ps [NOTE: .svg good for Illustrator and .ps broken(?)]')

    myparser.add_option('--xmin',dest='xmin',default=None,type='float',help='force plot range')
    myparser.add_option('--xmax',dest='xmax',default=None,type='float',help='force plot range')
    myparser.add_option('--ymin',dest='ymin',default=None,type='float',help='force plot range')
    myparser.add_option('--ymax',dest='ymax',default=None,type='float',help='force plot range')

    myparser.add_option('--todo',dest='todo',default=False, action='store_true',
                        help='Print todo list')

    import pmag_verbosity
    pmag_verbosity.addVerbosityOptions(myparser)

    (options,args) = myparser.parse_args()

    if options.todo:
        printTodo()
        sys.exit()

    if None != options.xmin and None == options.xmax: sys.exit("ERROR: must specify both min and max for X")
    if None == options.xmin and None != options.xmax: sys.exit("ERROR: must specify both min and max for X")
    if None != options.ymin and None == options.ymax: sys.exit("ERROR: must specify both min and max for Y")
    if None == options.ymin and None != options.ymax: sys.exit("ERROR: must specify both min and max for Y")

    if None != options.cpsFile:
        xCps = XRDcps(options.cpsFile,degreeOffset=options.offset,verbosity=options.verbosity)
        if not options.cps: options.esd = True # At least plot something!
        if options.show: xCps.plot(cps=options.cps,esd=options.esd,options=options)
        if None != options.plotFile: xCps.plot(options.plotFile,cps=options.cps,esd=options.esd,options=options) 

    # FIX: warning if nothing is done!
    
